function W = ERM_ARG(X, p)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% entropy rate minimization via AR driven by GGD process
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        mixtures;
% p:        AR order, p is a positive integer
% Output:
% W:        demixing matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference:
%
% Geng-Shen Fu, Ronald Phlypo, Matthew Anderson, Xi-Lin Li, and Tulay Adali,
% "Blind Source Separation by Entropy Rate Minimization," 
% IEEE Trans. on Signal Process., 2014, 62, 4245-4255.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Geng-Shen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
show_cost = 0;

useTrueParamFlag = 0;

%preprocessing
[N,T] = size(X);
[X, P] = pre_processing( X );

% initialization
% W = eye(N);
W = ICA_EBM(X);     % use ICA-EBM to provide the initial guess

tolerance = 1e-5;
mu = 1/6;
max_iter=200;
max_cost_increase = 3;
max_cost_decrease = 5;
cost_increase_counter = 0;
cost_decrease_counter=0;
W_last = W;
W_best = W;
min_cost=inf;

Xtmp=[zeros(N,p), X];
X_KK=zeros(N,T*(p+1));
for i=1:T
    X_KK(:,(i-1)*(p+1)+1:i*(p+1))=Xtmp(:,p+i:-1:i);
end;

for iter=1:max_iter
    Y=W*X;
    
    for n=1:N
        Yobj=iddata(Y(n,:)');
        mb = ar(Yobj,p,'yw');
        a(:,n)=polydata(mb);
        Yw(n,:) = filter(a(:,n), 1, Y(n,:)');
    end
    
    [Alpha, Beta]=GGDParamEstor(Yw);
    Cost(iter)=EntropyEstor(Alpha, Beta, W, N);
    
    %Updata W
    %Calculate vector gradient
    for i=1:N
        w_delta=VectorGradientEstor(i, W, X_KK, a(:,i), Yw(i,:), Alpha(i), Beta(i));
        w1 = W(i,:) - mu*w_delta;
        w1 = w1 / norm(w1);
        W(i,:) = w1;
    end;
    
    %Check convergence
    if Cost(iter) < min_cost
        cost_increase_counter = 0;
        min_cost = Cost(iter);
        W_best = W_last;
        cost_decrease_counter=cost_decrease_counter+1;
        if cost_decrease_counter>max_cost_decrease
            cost_decrease_counter=0;
            mu=mu*1.5;
        end;
    else
        cost_increase_counter = cost_increase_counter + 1;
        cost_decrease_counter=0;
    end
    min_cost_queue(iter) = min_cost;
    
    if cost_increase_counter > max_cost_increase
        W1 = W;
        W1_last = W_last;
        for n = 1 : N
            W1(n,:) = W1(n,:)/norm(W1(n,:));
            W1_last(n,:) = W1_last(n,:)/norm(W1_last(n,:));
        end
        if sum(abs(abs(diag(W1*W1_last'))-1)) < N*tolerance
            break;
        else
            mu = mu/2;
            W = W_best;
            W_last = W_best;
            cost_increase_counter = 0;
            continue;
        end
    end
    
    W_last = W;
end
W=W_best;
W = W*P;
    
if show_cost
    figure;
    plot([1:iter],Cost([1:iter]), '--ro');
    xlabel('Number of iterations');
    ylabel('Cost');
end
i=1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function I = EntropyEstor( Alpha, Beta, W, N )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate the cost, mutual information rate, of PFBSS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% N:        number of sources;
% W:        demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelations;
% K:        memory length;
% Output:
% I:        mutual information rate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin EntropyEstor!');
I=-log(abs(det(W)));
for n=1:N
    H=(1/Beta(n))-log(Beta(n)/2/Alpha(n)/gamma(1/Beta(n)));
    I=I+H;
end;
i=1;

function w_delta = VectorGradientEstor( n, W, X_KK, a, Xw, alpha, beta )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate demixing matrix nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Y:        reconstruction sources;
% W:        a row of demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
% Output:
% W_delta:  nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin GradientEstor!');
N=size(W,2);
T=size(Xw, 2);

Phi=sign(Xw).*(abs(Xw).^(beta-1))*(beta)/(alpha^beta);
Phi_a = kron(Phi',a);
w_delta=(X_KK*Phi_a/T)';

temp1 = randn(N, 1);
W_tilde = [W(1:n-1,:); W(n+1:N,:)];
h = temp1 - W_tilde'*inv(W_tilde*W_tilde')*W_tilde*temp1;
w = W(n,:);
w_delta = w_delta - h'/(w*h);
        
w_delta = w_delta - (w_delta*w')*w;
w_delta = w_delta / norm(w_delta);


function [Alpha, Beta] = GGDParamEstor( X )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate distribution parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        reconstruction sources;
% Output:
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin PDFParamEstor!');
[N T]=size(X);

for n=1:N
    %estimate shape parameter
    [Mu(n), Alpha(n), Beta(n)] = ggmle(X(n,:));
end;
i=1;


function [X, P] = pre_processing( X )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pre-processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        data;
% Output:
% X:        pre-processed data;
% P:        whitening matrix;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin pre_processing!');
[N,T] = size(X);
% remove DC
Xmean=mean(X,2);
X = X - Xmean*ones(1,T);

% spatio pre-whitening 1 
R = X*X'/T;                 
P = inv_sqrtmH(R);  %P = inv(sqrtm(R));
X = P*X;


function A = inv_sqrtmH( B )
[V,D] = eig(B);
d = diag(D);
d = 1./sqrt(d);
A = V*diag(d)*V';


function p0 = order_MAp(x, maxp0)
% return the estimated order of moving average process x 
% maxp0: maximum MA order
% p0: estimated order (Note: it is a random number)
% Program by Xi-Lin Li: lixilin@umbc.edu
p0 = 1;
while 1
      
    if iswhitep( x(ceil(rand*p0):p0:end) ) % ceil(rand*p0) is a random offset
        return;
    else
        p0=p0+1;
        if p0>maxp0
            p0=maxp0;
            return;
        end
    end
    
end

function s = iswhitep(x)
% hypothesis: white process vs colored process
% if accept white, return 1; otherwise, return 0
% Program by Xi-Lin Li: lixilin@umbc.edu
T=length(x);

% DL of white case
p=0;
dl0=0.5*log(mean(x.^2)) + 0.5*(0+1)*log(T)/T;

% DL of colored case with AR order 1
p=1;
x1 = [0,x(1:end-1)];
a = x*x1'/(x1*x1');
n = x-a*x1;
dl1 = 0.5*log(mean(n.^2)) + 0.5*(p+1)*log(T)/T;
    
s=(dl0<dl1);