function W = ERM_MG(X, varargin)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% entropy rate minimization via multivariate generalized Gaussian distribution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        mixtures;
% Optional input pairs and default values are given in list below:
%    'K','-1', ... % dimension for MGGD. -1: estimate automatically
%    'Beta',0, ... % shape parameter for MGGD. 0: estimate automatically
%    'OptMtd',0, ... % optimization method. 0: vector optimization 
%    'EstMtd',0, ... % MGGD parameter estimation method. 0: MoM
%    'InitMtd',0, ... % initialization. 0: EBM; other: ERBM-AR
%    'S',[], ... % true latent sources
% Output:
% W:        demixing matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reference:
%
% Geng-Shen Fu, Ronald Phlypo, Matthew Anderson, Xi-Lin Li, and Tulay Adali,
% "Blind Source Separation by Entropy Rate Minimization," 
% IEEE Trans. on Signal Process., 2014, 62, 4245-4255.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Geng-Shen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
show_cost = 0;

%set default parameters
Kparam=-1;       %Estimate K
Betaparam=0;     %Estimate Beta
matrixOpt=0;     %Vector optimization
estML=0;         %Estimate by MoM
initMethod=0;    %Initialize by EBM
S=[];            %No original signal

i=1;
while i<size(varargin,2)
    if strcmp(varargin{i},'K')
        Kparam=varargin{i+1};
    elseif strcmp(varargin{i},'Beta')
        Betaparam=varargin{i+1};
    elseif strcmp(varargin{i},'OptMtd')
        matrixOpt=varargin{i+1};
    elseif strcmp(varargin{i},'EstMtd')
        estML=varargin{i+1};
    elseif strcmp(varargin{i},'InitMtd')
        initMethod=varargin{i+1};
    elseif strcmp(varargin{i},'S')
        S=varargin{i+1};
    end
    i=i+2;
end

useTrueParamFlag = 0;

%preprocessing
[N,T] = size(X);
[X, P] = pre_processing( X );

% initialization
% W=eye(N);
if initMethod == 0
    W = ICA_EBM(X);     % use ICA-EBM to provide the initial guess
else
    W = ERBM_AR(X, 7);     % use FBSS_AR to provide the initial guess
end;

if prod(size(S)) ~= 0
    [BetaK, BetaK1, SigmaK, SigmaK1, K]=PDFParamEstor(S, Kparam, Betaparam, 0);
    Kcost = K;
    if estML == 1
    for i=1:N
        [BetaK(i), SigmaK(i,:,:)] = PDFParamEstorMLJointly( S(i,:), BetaK(i), squeeze(SigmaK(i,:,:)) );
        [BetaK1(i), SigmaK1(i,:,:)] = PDFParamEstorMLJointly( S(i,:), BetaK1(i), squeeze(SigmaK1(i,:,:)) );
    end
    end
    CostTrue=EntropyEstor(N, eye(N), BetaK, BetaK1, SigmaK, SigmaK1, K);
    if prod(BetaK-1) > 0
        disp('Both sub or sup Gaussian!!!');
    else
        useTrueParamFlag = 1;
    end
else
    [BetaK, BetaK1, SigmaK, SigmaK1, Kcost]=PDFParamEstor(W*X, Kparam, Betaparam, estML);
end

tolerance = 1e-5;
mu = 1/6;
max_iter=200;
max_cost_increase = 3;
max_cost_decrease = 5;
cost_increase_counter = 0;
cost_decrease_counter=0;
W_last = W;
W_best = W;
min_cost=inf;

for iter=1:max_iter
    Y=W*X;
    if estML ~= 1
    [BetaKcost, BetaK1cost, SigmaKcost, SigmaK1cost, Kcost]=PDFParamEstor(Y, Kparam, Betaparam, estML);
    else
    for i=1:N
        [BetaKcost(i), SigmaKcost(i,:,:)] = PDFParamEstorMLJointly( Y(i,:), BetaK(i), squeeze(SigmaK(i,:,:)) );
        [BetaK1cost(i), SigmaK1cost(i,:,:)] = PDFParamEstorMLJointly( Y(i,:), BetaK1(i), squeeze(SigmaK1(i,:,:)) );
    end
    end;
    if useTrueParamFlag == 0
        BetaK=BetaKcost;
        BetaK1=BetaK1cost;
        SigmaK=SigmaKcost;
        SigmaK1=SigmaK1cost;
        K=Kcost;
    end
    Cost(iter)=EntropyEstor(N, W, BetaKcost, BetaK1cost, SigmaKcost, SigmaK1cost, Kcost);
    
    %Updata W
    if matrixOpt == 1
        %Calculate nature gradient
        W_delta=MatrixGradientEstor(Y, W, BetaK, BetaK1, SigmaK, SigmaK1, K);
        W=W+mu*W_delta;
    else
        %Calculate vector gradient
        for i=1:N
            w_delta=VectorGradientEstor(i, X, Y(i,:), W, BetaK(i), BetaK1(i), squeeze(SigmaK(i,:,:)), squeeze(SigmaK1(i,:,:)), K);
            w1 = W(i,:) - mu*w_delta;
            w1 = w1 / norm(w1);
            W(i,:) = w1;
        end;
    end
   
    %Check convergence
    if Cost(iter) < min_cost
        cost_increase_counter = 0;
        min_cost = Cost(iter);
        W_best = W_last;
        cost_decrease_counter=cost_decrease_counter+1;
        if cost_decrease_counter>max_cost_decrease
            cost_decrease_counter=0;
            mu=mu*1.5;
        end;
    else
        cost_increase_counter = cost_increase_counter + 1;
        cost_decrease_counter=0;
    end
    min_cost_queue(iter) = min_cost;
    
    if cost_increase_counter > max_cost_increase
        W1 = W;
        W1_last = W_last;
        for n = 1 : N
            W1(n,:) = W1(n,:)/norm(W1(n,:));
            W1_last(n,:) = W1_last(n,:)/norm(W1_last(n,:));
        end
        if sum(abs(abs(diag(W1*W1_last'))-1)) < N*tolerance
            break;
        else
            mu = mu/2;
            W = W_best;
            W_last = W_best;
            cost_increase_counter = 0;
            continue;
        end
    end
    
    W_last = W;
end
W=W_best;
W = W*P;
    
if show_cost
    figure;
    plot([1:iter],Cost([1:iter]), '--ro');
    if useTrueParamFlag == 1
        hold on;
        plot([1:iter],CostTrue, '--bx');
    end;
    xlabel('Number of iterations');
    ylabel('Cost');
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function I = EntropyEstor( N, W, BetaK, BetaK1, SigmaK, SigmaK1, K )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Estimate the cost, mutual information rate, of PFBSS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% N:        number of sources;
% W:        demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelations;
% K:        memory length;
% Output:
% I:        mutual information rate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin EntropyEstor!');
err=0;
I=-log(abs(det(W)));
for n=1:N
    [H errTmp] = getGGDEntropy(BetaK(n), K, squeeze(SigmaK(n,:,:)));
    err=err+errTmp;
    I=I+H;
    if K>1
        [H errTmp] = getGGDEntropy(BetaK1(n), K-1, squeeze(SigmaK1(n,:,:)));
        err=err+errTmp;
        I=I-H;
    end;
end;
if err>0
    I=inf;
end;


function Sigma = getCovbyACS( R, Beta )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get covariance matrix from autocorrelation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Beta:     shape parameters;
% R:        sample autocorrelations;
% Output:
% Sigma:    covariance matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K=size(R,2);
%estimation of covariance matrix by MM.
Sigma=toeplitz(R)*K*gamma(K/2/Beta)/(2^(1/Beta))/gamma((K+2)/2/Beta);
% Sigma=toeplitz(R);    %fugs for debug
%estimation of covariance matrix by ML.



function [H, err] = getGGDEntropy( Beta, K, Sigma )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% get entropy for MGGD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Beta:     shape parameters;
% K:        memory length;
% Sigma:    covariance matrix;
% Output:
% H:        entropy
% err:      error flag for Sigma is not PD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
err=0;
H=log(abs(det(Sigma)))/2+(K/2/Beta)-log(Beta*gamma(K/2)/(2^(K/2/Beta))/(pi^(K/2))/gamma(K/2/Beta));
if det(Sigma)<0
    disp('Sigma < 0 !!!');
    err=1;
end;


function w_delta = VectorGradientEstor( n_w, X, Y, W, BetaK, BetaK1, SigmaK, SigmaK1, K )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate demixing matrix nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Y:        reconstruction sources;
% W:        a row of demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
% Output:
% W_delta:  nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin GradientEstor!');
[N,T]=size(X);

Sigma_inv=inv(SigmaK);

yConvTmp=convmtx(Y,K);
yConv=yConvTmp(end:-1:1,K:end-K+1)';
yConvSigma=yConv*Sigma_inv;
for i=1:N
    xConvTmp=convmtx(X(i,:),K);
    xConv(i,:,:)=xConvTmp(end:-1:1,K:end-K+1)';
end;

if K==1
    xConv=reshape(xConv,N,K,T-K+1);
end;

mahaDist=(sum(yConvSigma.*yConv,2).^(BetaK-1))*BetaK;
for j=1:N
    w_delta1(j)=mean(mahaDist.*(sum(yConvSigma.*(squeeze(xConv(j,:,:))),2)));
end;
w_delta=w_delta1;

if K>1
yConvTmp=[];
yConv=[];
xConvTmp=[];
xConv=[];
yConvSigma=[];
Sigma_inv=[];

Sigma_inv=inv(SigmaK1);

yConvTmp=convmtx(Y,K-1);
yConv=yConvTmp(end:-1:1,K-1:end-K+2)';
yConvSigma=yConv*Sigma_inv;
for i=1:N
    xConvTmp=convmtx(X(i,:),K-1);
    xConv(i,:,:)=xConvTmp(end:-1:1,K-1:end-K+2)';
end;

if K==2
    xConv=reshape(xConv,N,K-1,T-K+2);
end;

mahaDist=(sum(yConvSigma.*yConv,2).^(BetaK1-1))*BetaK1;
for j=1:N
    w_delta2(j)=mean(mahaDist.*(sum(yConvSigma.*(squeeze(xConv(j,:,:))),2)));
end;

w_delta=w_delta-w_delta2;
end;

temp1 = randn(N, 1);
W_tilde = [W(1:n_w-1,:); W(n_w+1:N,:)];
h = temp1 - W_tilde'*inv(W_tilde*W_tilde')*W_tilde*temp1;
w = W(n_w,:);
w_delta = w_delta - h'/(w*h);
        
w_delta = w_delta - (w_delta*w')*w;
w_delta = w_delta / norm(w_delta);


function W_delta = MatrixGradientEstor( Y, W, BetaK, BetaK1, SigmaK, SigmaK1, K )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate demixing matrix nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Y:        reconstruction sources;
% W:        demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
% Output:
% W_delta:  nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin GradientEstor!');
[N,T]=size(Y);

for n=1:N
    Sigma_inv(n,:,:)=inv(squeeze(SigmaK(n,:,:)));
end;

for i=1:N
    yConvTmp=convmtx(Y(i,:),K);
    yConv(i,:,:)=yConvTmp(end:-1:1,K:end-K+1)';
    yConvSigma(i,:,:)=squeeze(yConv(i,:,:))*squeeze(Sigma_inv(i,:,:));
end;

if K==1
    yConv=reshape(yConv,N,K,T-K+1);
    yConvSigma=reshape(yConvSigma,N,K,T-K+1);
end;

for i=1:N
    mahaDist=(sum(squeeze(yConvSigma(i,:,:)).*(squeeze(yConv(i,:,:))),2).^(BetaK(i)-1))*BetaK(i);
    for j=1:N
        W_delta1(i,j)=mean(mahaDist.*(sum(squeeze(yConvSigma(i,:,:)).*(squeeze(yConv(j,:,:))),2)));
    end;
end;
W_delta=eye(N)-W_delta1;

if K>1
yConvTmp=[];
yConv=[];
yConvSigma=[];
Sigma_inv=[];
for n=1:N
    Sigma_inv(n,:,:)=inv(squeeze(SigmaK1(n,:,:)));
end;

for i=1:N
    yConvTmp=convmtx(Y(i,:),K-1);
    yConv(i,:,:)=yConvTmp(end:-1:1,K-1:end-K+2)';
    yConvSigma(i,:,:)=squeeze(yConv(i,:,:))*squeeze(Sigma_inv(i,:,:));
end;

if K==2
    yConv=reshape(yConv,N,K-1,T-K+2);
    yConvSigma=reshape(yConvSigma,N,K-1,T-K+2);
end;

for i=1:N
    mahaDist=(sum(squeeze(yConvSigma(i,:,:)).*(squeeze(yConv(i,:,:))),2).^(BetaK1(i)-1))*BetaK1(i);
    for j=1:N
        W_delta2(i,j)=mean(mahaDist.*(sum(squeeze(yConvSigma(i,:,:)).*(squeeze(yConv(j,:,:))),2)));
    end;
end;
W_delta=W_delta+W_delta2;
end;
W_delta=W_delta*W;

for n=1:N
    W_delta(n,:) = W_delta(n,:) - W_delta(n,:)*(W(n,:)')*W(n,:)/(W(n,:)*(W(n,:)'));
end;


function W_delta = MatrixGradientEstor2( Y, W, Beta, R, K )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate demixing matrix nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% Y:        reconstruction sources;
% W:        demixing matrix;
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
% Output:
% W_delta:  nature gradient
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[N,T]=size(Y);

for n=1:N
    Sigma_inv(n,:,:)=inv(getCovbyACS(R(n,:), Beta(n)));
end;

W_delta1=zeros(N,N);
for t=1:T-K+1
    idx=t:t+K-1;
    Varphi=[];
    for n=1:N
        Varphi_n=Beta(n)*((Y(n,idx)*squeeze(Sigma_inv(n,:,:))*(Y(n,idx)'))^(Beta(n)-1))*Y(n,idx)*squeeze(Sigma_inv(n,:,:));
        Varphi=[Varphi; Varphi_n];
    end;
    W_delta1=W_delta1+Varphi*((Y(:,idx))');
end;
W_delta1=W_delta1/(T-K+1);
W_delta=eye(N)-W_delta1;

if K>1
Sigma_inv=[];
for n=1:N
    Sigma_inv(n,:,:)=inv(getCovbyACS(R(n,1:end-1), Beta(n)));
end;

W_delta2=zeros(N,N);
for t=1:T-K+2
    idx=t:t+K-2;
    Varphi=[];
    for n=1:N
        Varphi_n=Beta(n)*((Y(n,idx)*squeeze(Sigma_inv(n,:,:))*(Y(n,idx)'))^(Beta(n)-1))*Y(n,idx)*squeeze(Sigma_inv(n,:,:));
        Varphi=[Varphi; Varphi_n];
    end;
    W_delta2=W_delta2+Varphi*((Y(:,idx))');
end;
W_delta2=W_delta2/(T-K+2);
W_delta=W_delta+W_delta2;
end;

W_delta=W_delta*W;

for n=1:N
    W_delta(n,:) = W_delta(n,:) - W_delta(n,:)*(W(n,:)')*W(n,:)/(W(n,:)*(W(n,:)'));
end;


function [BetaK, BetaK1, SigmaK, SigmaK1, K] = PDFParamEstor( X, Kparam, Betaparam, estML )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate distribution parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        reconstruction sources;
% Output:
% Beta:     shape parameters;
% R:        sample autocorrelation sequence;
% K:        memory length;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin PDFParamEstor!');
[N T]=size(X);

if Kparam ~= -1
    K=Kparam+1;
else
%estimate memory length
for n=1:N
    k(n) = order_MAp(X(n,:), floor(T/20));
%     k(n) = 7;    %fugs for debug
end;
K=ceil(mean(k));
% K=2;
end


for n=1:N
    %estimate shape parameter
    if Betaparam == 0
    [mu(n), alpha(n), Beta(n)] = ggmle(X(n,:));
    Beta(n)=Beta(n)/2;
    else
        Beta(n)=Betaparam;
    end
%     Beta(n)=1;    %fugs for debug
    
    %estimate autocorrelation sequence
    acs=xcorr(X(n,:),K-1,'unbiased');
    R(n,:)=acs(K:end);
    if estML== 1 && K>1
        [BetaK(n) SigmaK(n,:,:)] = PDFParamEstorML( X(n,:), Beta(n), R(n,:) );
        [BetaK1(n) SigmaK1(n,:,:)] = PDFParamEstorML( X(n,:), Beta(n), R(n,1:end-1) );
    else
        SigmaK(n,:,:) = getCovbyACS( R(n,:), Beta(n) );
        if K>1
            SigmaK1(n,:,:) = getCovbyACS( R(n,1:end-1), Beta(n) );
        else
            SigmaK1(n,:,:) = 0;
        end
        BetaK(n)=Beta(n);
        BetaK1(n)=Beta(n);
    end;
end;
i=1;


function [BetaML SigmaML] = PDFParamEstorML( X, BetaMM, R )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate distribution parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        reconstruction sources;
% Beta:     shape parameter;
% R:        autocorrelation sequence;
% Output:
% Sigma:    covariance matrix;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tolSigma = 1e-3;
tolBeta = 1e-5;
max_iter=20;
T=size(X, 2);
K=size(R, 2);
cost_last=K*K;

xConvTmp=convmtx(X,K);
xConv=xConvTmp(end:-1:1,K:end-K+1)';
SigmaMM = getCovbyACS( R, BetaMM );
SigmaML=SigmaMM;
SigmaML_last = SigmaMM;
mahaDist=sum((xConv*inv(SigmaMM)).*xConv, 2);

f = @(Beta) ( Beta/2*(log(mahaDist)'*(mahaDist.^Beta)) - (T-K+1)*(K/2/Beta * (log(2)+psi(K/2/Beta)) + 1));
BetaL = .1;
BetaR = 10;
iter =0;
while (BetaR-BetaL)> tolBeta
    iter = iter+1;
    BetaML = BetaL + (BetaR-BetaL)/2;
    if f(BetaML)*f(BetaR) < 0, BetaL = BetaML; else BetaR = BetaML; end
end
% [BetaMM BetaML iter]

% for BetaI = 1:100
%     Betaf(BetaI) = f(BetaI/10);
% end
% figure, plot((1:100)/10,Betaf)

for iter=1:max_iter
%     4*gamma(4/2/Beta)/(2^(1/Beta))/gamma(6/2/Beta)
    mahaDist=BetaML*sum((xConv*inv(SigmaML)).*xConv, 2).^(BetaML-1);
    SigmaML=(xConv')*(repmat(mahaDist,1,K).*xConv)/(T-K+1);
    cost=sum(sum(abs(SigmaML-SigmaML_last)));
    if cost <= cost_last
        if cost < cost_last-(tolSigma*K*K)
            SigmaML_last = SigmaML;
            cost_last = cost;
        else
            break;
        end
    else 
        SigmaML=SigmaML_last;
        break;
    end
end;
% [cost iter]
% SigmaMM
% SigmaML
% i=1;



function [BetaML SigmaML] = PDFParamEstorMLJointly( X, Beta, Sigma )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% estimate distribution parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        reconstruction sources;
% Beta:     shape parameter;
% R:        autocorrelation sequence;
% Output:
% Sigma:    covariance matrix;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tolSigma = 1e-3;
tolBeta = 1e-5;
max_iter=20;
T=size(X, 2);
K=size(Sigma, 2);
cost_last=K*K;
cost_prev=(K*K+1)*2;

xConvTmp=convmtx(X,K);
xConv=xConvTmp(end:-1:1,K:end-K+1)';


SigmaML=Sigma;
BetaML=Beta;
SigmaML_prev = SigmaML;
BetaML_prev = BetaML;

for iterJoint = 1:max_iter
SigmaML_last = SigmaML;
mahaDist=sum((xConv*inv(SigmaML)).*xConv, 2);

f = @(Beta) ( Beta/2*(log(mahaDist)'*(mahaDist.^Beta)) - (T-K+1)*(K/2/Beta * (log(2)+psi(K/2/Beta)) + 1));

BetaL = .1;
BetaR = 10;
iter =0;
while (BetaR-BetaL)> tolBeta
    iter = iter+1;
    BetaML = BetaL + (BetaR-BetaL)/2;
    if f(BetaML)*f(BetaR) < 0, BetaL = BetaML; else BetaR = BetaML; end
end
% [Beta BetaML iter]

% for BetaI = 1:100
%     Betaf(BetaI) = f(BetaI/10);
% end
% figure, plot((1:100)/10,Betaf)

for iter=1:max_iter
%     4*gamma(4/2/Beta)/(2^(1/Beta))/gamma(6/2/Beta)
    mahaDist=BetaML*sum((xConv*inv(SigmaML)).*xConv, 2).^(BetaML-1);
    SigmaML=(xConv')*(repmat(mahaDist,1,K).*xConv)/(T-K+1);
    cost=sum(sum(abs(SigmaML-SigmaML_last)));
    if cost <= cost_last
        if cost < cost_last-(tolSigma*K*K)
            SigmaML_last = SigmaML;
            cost_last = cost;
        else
            break;
        end
    else 
        SigmaML=SigmaML_last;
        break;
    end
end;
% [cost iter]
% Sigma
% SigmaML

    cost=sum(sum(abs(SigmaML-SigmaML_prev))) + abs(BetaML-BetaML_prev)*2;
    if cost <= cost_prev
        if cost < cost_prev-(tolSigma*K*K)-2*tolBeta
            SigmaML_prev = SigmaML;
            BetaML_prev = BetaML;
            cost_prev = cost;
        else
            break;
        end
    else 
        SigmaML=SigmaML_prev;
        BetaML=BetaML_prev;
        break;
    end

end;
% [Beta BetaML cost iterJoint]
% Sigma
% SigmaML
i=1;


function [X, P] = pre_processing( X )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% pre-processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X:        data;
% Output:
% X:        pre-processed data;
% P:        whitening matrix;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Coded by Gengshen Fu, fugengs1@umbc.edu
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% disp('begin pre_processing!');
[N,T] = size(X);
% remove DC
Xmean=mean(X,2);
X = X - Xmean*ones(1,T);

% spatio pre-whitening 1 
R = X*X'/T;                 
P = inv_sqrtmH(R);  %P = inv(sqrtm(R));
X = P*X;


function A = inv_sqrtmH( B )
[V,D] = eig(B);
d = diag(D);
d = 1./sqrt(d);
A = V*diag(d)*V';


function p0 = order_MAp(x, maxp0)
% return the estimated order of moving average process x 
% maxp0: maximum MA order
% p0: estimated order (Note: it is a random number)
% Program by Xi-Lin Li: lixilin@umbc.edu
p0 = 1;
while 1
      
    if iswhitep( x(ceil(rand*p0):p0:end) ) % ceil(rand*p0) is a random offset
        return;
    else
        p0=p0+1;
        if p0>maxp0
            p0=maxp0;
            return;
        end
    end
    
end

function s = iswhitep(x)
% hypothesis: white process vs colored process
% if accept white, return 1; otherwise, return 0
% Program by Xi-Lin Li: lixilin@umbc.edu
T=length(x);

% DL of white case
p=0;
dl0=0.5*log(mean(x.^2)) + 0.5*(0+1)*log(T)/T;

% DL of colored case with AR order 1
p=1;
x1 = [0,x(1:end-1)];
a = x*x1'/(x1*x1');
n = x-a*x1;
dl1 = 0.5*log(mean(n.^2)) + 0.5*(p+1)*log(T)/T;
    
s=(dl0<dl1);